searchData={"items":[{"type":"module","title":"Earmark","doc":"","ref":"Earmark.html"},{"type":"module","title":"Earmark - Earmark","doc":"#","ref":"Earmark.html#module-earmark"},{"type":"module","title":"Abstract Syntax Tree and Rendering - Earmark","doc":"The AST generation has now been moved out to [`Earmark.Parser`](https://github.com/robertdober/earmark_parser)\nwhich is installed as a dependency.\n\nThis brings some changes to this documentation and also deprecates the usage of `Earmark.as_ast`\n\nEarmark takes care of rendering the AST to HTML, exposing some AST Transformation Tools and providing a CLI as escript.\n\nTherefore you will not find a detailed description of the supported Markdown here anymore as this is done in\n[here](https://hexdocs.pm/earmark_parser/Earmark.Parser.html)\n\n\n\n##","ref":"Earmark.html#module-abstract-syntax-tree-and-rendering"},{"type":"module","title":"Earmark.as_ast - Earmark","doc":"WARNING: This is just a proxy towards `Earmark.Parser.as_ast` and is deprecated, it will be removed in version 1.5!\n\nReplace your calls to `Earmark.as_ast` with `EarmarkParse.as_ast` as soon as possible.\n\n**N.B.** If all you use is `Earmark.as_ast` consider _only_ using `Earmark.Parser`.\n\nAlso please refer yourself to the documentation of [`Earmark.Parser`](https://hexdocs.pm/earmark_parser/Earmark.Parser.html)\n\n\nThe function is described below and the other two API functions `as_html` and `as_html!` are now based upon\nthe structure of the result of `as_ast`.\n\n    {:ok, ast, []}                   = Earmark.Parser.as_ast(markdown)\n    {:ok, ast, deprecation_messages} = Earmark.Parser.as_ast(markdown)\n    {:error, ast, error_messages}    = Earmark.Parser.as_ast(markdown)\n\n##","ref":"Earmark.html#module-earmark-as_ast"},{"type":"module","title":"Earmark.as_html - Earmark","doc":"{:ok, html_doc, []}                   = Earmark.as_html(markdown)\n    {:ok, html_doc, deprecation_messages} = Earmark.as_html(markdown)\n    {:error, html_doc, error_messages}    = Earmark.as_html(markdown)\n\n##","ref":"Earmark.html#module-earmark-as_html"},{"type":"module","title":"Earmark.as_html! - Earmark","doc":"html_doc = Earmark.as_html!(markdown, options)\n\nFormats the error_messages returned by `as_html` and adds the filename to each.\nThen prints them to stderr and just returns the html_doc\n\n##","ref":"Earmark.html#module-earmark-as_html"},{"type":"module","title":"Options - Earmark","doc":"Options can be passed into as `as_html/2` or `as_html!/2` according to the documentation.\nA keyword list with legal options (c.f. `Earmark.Options`) or an `Earmark.Options` struct are accepted.\n\n    {status, html_doc, errors} = Earmark.as_html(markdown, options)\n    html_doc = Earmark.as_html!(markdown, options)\n    {status, ast, errors} = Earmark.Parser.as_ast(markdown, options)\n\n#","ref":"Earmark.html#module-options"},{"type":"module","title":"Rendering - Earmark","doc":"All options passed through to `Earmark.Parser.as_ast` are defined therein, however some options concern only\nthe rendering of the returned AST\n\nThese are:\n\n* `compact_output:` defaults to `false`\n\nNormally `Earmark` aims to produce _Human Readable_ output.\n\nThis will give results like these:\n\n    iex(1)> markdown = \"# Hello\\nWorld\"\n    ...(1)> Earmark.as_html!(markdown, compact_output: false)\n    \" \\nHello \\n \\nWorld \\n\"\n\n\nBut sometimes whitespace is not desired:\n\n    iex(2)> markdown = \"# Hello\\nWorld\"\n    ...(2)> Earmark.as_html!(markdown, compact_output: true)\n    \" Hello  World \"\n\nBe cautions though when using this options, lines will become loooooong.\n\n\n#### `escape:` defaulting to `true`\n\nIf set HTML will be properly escaped\n\n      iex(3)> markdown = \"Hello World\"\n      ...(3)> Earmark.as_html!(markdown)\n      \" \\nHello&lt;br&gt;World \\n\"\n\nHowever disabling `escape:` gives you maximum control of the created document, which in some\ncases (e.g. inside tables) might even be necessary\n\n      iex(4)> markdown = \"Hello World\"\n      ...(4)> Earmark.as_html!(markdown, escape: false)\n      \" \\nHello World \\n\"\n\n#### `inner_html:` defaulting to `false`\n\nThis is especially useful inside templates, when a block element will disturb the layout as\nin this case\n\n```html\n   \n   \n```\n\nBy means of the `inner_html` option the disturbing paragraph can be removed from `as_html!`'s\noutput\n\n      iex(5)> markdown = \"Hello World\"\n      ...(5)> Earmark.as_html!(markdown, escape: false, inner_html: true)\n      \"Hello World\\n\"\n\n**N.B.** that this applies only to top level paragraphs, as can be seen here\n\n      iex(6)> markdown = \"- Item\\n\\nPara\"\n      ...(6)> Earmark.as_html!(markdown, inner_html: true)\n      \" \\n   \\nItem   \\n \\nPara\\n\"\n\n\n* `postprocessor:` defaults to nil\n\nBefore rendering, the AST is transformed by a postprocessor.\nFor details, see the description of `Earmark.Transform.map_ast` below which will accept the same postprocessor.\nAs a matter of fact, specifying `postprocessor: fun` is conceptually the same as\n\n```elixir\n          markdown\n          |> Earmark.Parser.as_ast\n          |> Earmark.Transform.map_ast(fun)\n          |> Earmark.Transform.transform\n```\n\nwith all the necessary bookkeeping for options and messages\n\n* `renderer:` defaults to `Earmark.HtmlRenderer`\n\n  The module used to render the final document.\n\n#### `smartypants:` defaulting to `true`\n\nIf set the following replacements will be made during rendering of inline text\n\n    \"---\" → \"—\"\n    \"--\" → \"–\"\n    \"' → \"’\"\n    ?\" → \"”\"\n    \"...\" → \"…\"\n\n#","ref":"Earmark.html#module-rendering"},{"type":"module","title":"Command line - Earmark","doc":"```sh\n    $ mix escript.build\n    $ ./earmark file.md\n```\n\nSome options defined in the `Earmark.Options` struct can be specified as command line switches.\n\nUse\n\n```sh\n    $ ./earmark --help\n```\n\nto find out more, but here is a short example\n\n```sh\n    $ ./earmark --smartypants false --code-class-prefix \"a- b-\" file.md\n```\n\nwill call\n\n```sh\n    Earmark.as_html!( ..., %Earmark.Options{smartypants: false, code_class_prefix: \"a- b-\"})\n```\n\n#","ref":"Earmark.html#module-command-line"},{"type":"module","title":"Timeouts - Earmark","doc":"By default, that is if the `timeout` option is not set Earmark uses parallel mapping as implemented in `Earmark.pmap/2`,\nwhich uses `Task.await` with its default timeout of 5000ms.\n\nIn rare cases that might not be enough.\n\nBy indicating a longer `timeout` option in milliseconds Earmark will use parallel mapping as implemented in `Earmark.pmap/3`,\nwhich will pass `timeout` to `Task.await`.\n\nIn both cases one can override the mapper function with either the `mapper` option (used if and only if `timeout` is nil) or the\n`mapper_with_timeout` function (used otherwise).\n\nFor the escript only the `timeout` command line argument can be used.\n\n#","ref":"Earmark.html#module-timeouts"},{"type":"module","title":"Security - Earmark","doc":"Please be aware that Markdown is not a secure format. It produces\nHTML from Markdown and HTML. It is your job to sanitize and or\nfilter the output of `Earmark.as_html` if you cannot trust the input\nand are to serve the produced HTML on the Web.","ref":"Earmark.html#module-security"},{"type":"function","title":"Earmark.as_ast/2","doc":"DEPRECATED call `Earmark.Parser.as_ast` instead","ref":"Earmark.html#as_ast/2"},{"type":"function","title":"Earmark.as_ast!/2","doc":"","ref":"Earmark.html#as_ast!/2"},{"type":"function","title":"Earmark.as_html/2","doc":"","ref":"Earmark.html#as_html/2"},{"type":"function","title":"Earmark.as_html!/2","doc":"","ref":"Earmark.html#as_html!/2"},{"type":"function","title":"Earmark.from_file!/2","doc":"A convenience method that *always* returns an HTML representation of the markdown document passed in.\nIn case of the presence of any error messages they are printed to stderr.\n\nOtherwise it behaves exactly as `as_html`.","ref":"Earmark.html#from_file!/2"},{"type":"function","title":"Earmark.pmap/3","doc":"","ref":"Earmark.html#pmap/3"},{"type":"function","title":"Earmark.transform/2","doc":"","ref":"Earmark.html#transform/2"},{"type":"function","title":"Earmark.version/0","doc":"Accesses current hex version of the `Earmark` application. Convenience for\n  `iex` usage.","ref":"Earmark.html#version/0"},{"type":"type","title":"Earmark.ast/0","doc":"","ref":"Earmark.html#t:ast/0"},{"type":"type","title":"Earmark.ast_attribute/0","doc":"","ref":"Earmark.html#t:ast_attribute/0"},{"type":"type","title":"Earmark.ast_attribute_name/0","doc":"","ref":"Earmark.html#t:ast_attribute_name/0"},{"type":"type","title":"Earmark.ast_attribute_value/0","doc":"","ref":"Earmark.html#t:ast_attribute_value/0"},{"type":"type","title":"Earmark.ast_attributes/0","doc":"","ref":"Earmark.html#t:ast_attributes/0"},{"type":"type","title":"Earmark.ast_meta/0","doc":"","ref":"Earmark.html#t:ast_meta/0"},{"type":"type","title":"Earmark.ast_node/0","doc":"","ref":"Earmark.html#t:ast_node/0"},{"type":"type","title":"Earmark.ast_tag/0","doc":"","ref":"Earmark.html#t:ast_tag/0"},{"type":"type","title":"Earmark.ast_tuple/0","doc":"","ref":"Earmark.html#t:ast_tuple/0"},{"type":"module","title":"Earmark.AstTools","doc":"Tools for AST manipulation","ref":"Earmark.AstTools.html"},{"type":"function","title":"Earmark.AstTools.find_att_in_node/2","doc":"Convenience function to access an attribute\n\n    iex(4)> find_att_in_node({\"a\", [{\"class\", \"link\"}], [], %{}}, \"class\")\n    \"link\"\n\n    iex(5)> find_att_in_node({\"a\", [{\"class\", \"link\"}], [], %{}}, \"target\")\n    nil\n\n    iex(6)> find_att_in_node({\"a\", [{\"class\", \"link\"}], [], %{}}, \"target\", :default)\n    :default\n\n    iex(7)> find_att_in_node([{\"class\", \"link\"}], \"class\")\n    \"link\"\n\n    iex(8)> find_att_in_node([{\"class\", \"link\"}], \"target\")\n    nil\n\n    iex(9)> find_att_in_node([{\"class\", \"link\"}], \"target\", :default)\n    :default","ref":"Earmark.AstTools.html#find_att_in_node/2"},{"type":"function","title":"Earmark.AstTools.find_att_in_node/3","doc":"","ref":"Earmark.AstTools.html#find_att_in_node/3"},{"type":"function","title":"Earmark.AstTools.merge_atts/2","doc":"A helper to merge attributes in their cannonical representation\n\n\n    iex(1)> merge_atts([{\"href\", \"url\"}], target: \"_blank\")\n    [{\"href\", \"url\"}, {\"target\", \"_blank\"}]\n\n    iex(2)> merge_atts([{\"href\", \"url\"}, {\"target\", \"nonsense\"}], %{\"target\" => \"_blank\"})\n    [{\"href\", \"url\"}, {\"target\", \"_blank nonsense\"}]\n\n    iex(3)>  merge_atts([{\"href\", \"url\"}, {\"target\", \"nonsense\"}, {\"alt\", \"nowhere\"}],\n    ...(3)>              [{\"target\", \"_blank\"}, title: \"where?\"])\n    [{\"alt\", \"nowhere\"}, {\"href\", \"url\"}, {\"target\", \"_blank nonsense\"}, {\"title\", \"where?\"}]","ref":"Earmark.AstTools.html#merge_atts/2"},{"type":"function","title":"Earmark.AstTools.merge_atts_in_node/2","doc":"A convenience function that extracts the original attributes to be merged with new attributes\nand puts the result into the node again\n\n    iex(10)> merge_atts_in_node({\"img\", [{\"src\", \"there\"}, {\"alt\", \"there\"}], [], %{some: \"meta\"}}, alt: \"here\")\n    {\"img\", [{\"alt\", \"here there\"}, {\"src\", \"there\"}], [], %{some: \"meta\"}}","ref":"Earmark.AstTools.html#merge_atts_in_node/2"},{"type":"function","title":"Earmark.AstTools.node_only_fn/1","doc":"Wrap a function that can only be called on nodes\n\n    iex(11)> f = fn {t, _, _, _} -> t end\n    ...(11)> f_ = node_only_fn(f)\n    ...(11)> {f_.({\"p\", [], [], %{}}), f_.(\"text\")}\n    {\"p\", \"text\"}","ref":"Earmark.AstTools.html#node_only_fn/1"},{"type":"module","title":"Earmark.Cli","doc":"The Earmark CLI\n\nEntry point of the escript, this is the **only** point that does IO with output and it uses the `Earmark.File` module which\nis the **only** point that does IO with input.","ref":"Earmark.Cli.html"},{"type":"function","title":"Earmark.Cli.main/1","doc":"This is the entry point of the escript","ref":"Earmark.Cli.html#main/1"},{"type":"function","title":"Earmark.Cli.run!/1","doc":"A convenience wrapper around `Earmark.Cli.Implementation.run/1` it will return `str` in case `{:stdio, str}` is returned\nor raise an error otherwise\n\n    iex(0)> run!([\"test/fixtures/short1.md\"])\n    \" \\nHeadline1 \\n \\n \\nHeadline2 \\n\"","ref":"Earmark.Cli.html#run!/1"},{"type":"module","title":"Earmark.Cli.Implementation","doc":"Functional (with the exception of reading input files with `Earmark.File`) interface to the CLI\nreturning the device and the string to be output.","ref":"Earmark.Cli.Implementation.html"},{"type":"function","title":"Earmark.Cli.Implementation.run/1","doc":"allows functional access to the CLI API, does everything `Earmark.Cli.main` does without outputting the result\n\nReturns a tuple of the device to write to (`:stdio|:stderr`) and the content to be written\n\nExample: Bad file\n\niex(0)> run([\"no-such--file--ayK7k\"])\n{:stderr, \"Cannot open no-such--file--ayK7k, reason: enoent\"}\n\nExample: Good file\n\niex(1)> {:stdio, html} = run([\"test/fixtures/short1.md\"])\n...(1)> html\n\" \\nHeadline1 \\n \\n \\nHeadline2 \\n\"\n\nExample: Using EEx\n\niex(2)> {:stdio, html} = run([\"--eex\", \"--gfm\", \"--code-class-prefix\", \"alpha\", \"--timeout\", \"12000\", \"test/fixtures/short2.md.eex\"])\n...(2)> html\n\" \\nShort2 \\n \\n Short3  \\n \\n\"\n\n\nExample: Using an EEx template first\n\niex(3)> run([\"--template\", \"test/fixtures/eex_first.html.eex\"])\n{:stdio, \" \\n   \\nShort2 \\n \\n Short3  \\n \\n\\n \\n\"}","ref":"Earmark.Cli.Implementation.html#run/1"},{"type":"module","title":"Earmark.EarmarkParserProxy","doc":"This acts as a proxy to adapt to changes in `Earmark.Parser`'s API\n\nIf no changes are needed it can delegate `as_ast` to `Earmark.Parser`\n\nIf changes are needed they will be realised in this modules `as_ast`\nfunction.\n\nFor that reason `Earmark.Parser.as_ast/*` **SHALL NOT** be invoked\nanywhere else in this code base","ref":"Earmark.EarmarkParserProxy.html"},{"type":"function","title":"Earmark.EarmarkParserProxy.as_ast/2","doc":"An adapter to `Earmark.Parser.as_ast/*`","ref":"Earmark.EarmarkParserProxy.html#as_ast/2"},{"type":"module","title":"Earmark.Helpers","doc":"","ref":"Earmark.Helpers.html"},{"type":"function","title":"Earmark.Helpers.replace/4","doc":"`Regex.replace` with the arguments in the correct order","ref":"Earmark.Helpers.html#replace/4"},{"type":"module","title":"Earmark.Internal","doc":"All public functions that are internal to Earmark, so that **only** external API\nfunctions are public in `Earmark`","ref":"Earmark.Internal.html"},{"type":"function","title":"Earmark.Internal.as_ast!/2","doc":"A wrapper to extract the AST from a call to `Earmark.Parser.as_ast` if a tuple `{:ok, result, []}` is returned,\nraise errors otherwise\n\n    iex(1)> as_ast!([\"Hello %% annotated\"], annotations: \"%%\")\n    [{\"p\", [], [\"Hello \"], %{annotation: \"%% annotated\"}}]\n\n    iex(2)> as_ast!(\"===\")\n    ** (Earmark.Error) [{:warning, 1, \"Unexpected line ===\"}]","ref":"Earmark.Internal.html#as_ast!/2"},{"type":"function","title":"Earmark.Internal.as_html!/2","doc":"","ref":"Earmark.Internal.html#as_html!/2"},{"type":"function","title":"Earmark.Internal.from_file!/2","doc":"This is a convenience method to read a file or pass it to `EEx.eval_file` if its name\nends in  `.eex`\n\nThe returned string is then passed to `as_html` this is used in the escript now and allows\nfor a simple inclusion mechanism, as a matter of fact an `include` function is passed","ref":"Earmark.Internal.html#from_file!/2"},{"type":"function","title":"Earmark.Internal.include/2","doc":"A utility function that will be passed as a partial capture to `EEx.eval_file` by\nproviding a value for the `options` parameter\n\n```elixir\n    EEx.eval(..., include: &include(&1, options))\n```\n\nthusly allowing\n\n```eex\n   \n```\n\nwhere `some file`  can be a relative path starting with `\"./\"`\n\nHere is an example using [these fixtures](https://github.com/pragdave/earmark/tree/master/test/fixtures)\n\n    iex(3)> include(\"./include/basic.md.eex\", file: \"test/fixtures/does_not_matter\")\n    \"# Headline Level 1\\n\"\n\nAnd here is how it is used inside a template\n\n    iex(4)> options = [file: \"test/fixtures/does_not_matter\"]\n    ...(4)> EEx.eval_string(~s{ }, include: &include(&1, options))\n    \"# Headline Level 1\\n\"","ref":"Earmark.Internal.html#include/2"},{"type":"module","title":"Earmark.Options","doc":"This is a superset of the options that need to be passed into `Earmark.Parser.as_ast/2`\n\nThe following options are proper to `Earmark` only and therefore explained in detail\n\n- `compact_output`: boolean indicating to avoid indentation and minimize whitespace\n- `eex`: Allows usage of an `EEx` template to be expanded to markdown before conversion\n- `file`: Name of file passed in from the CLI\n- `line`: 1 but might be set to an offset for better error messages in some integration cases\n- `smartypants`: boolean use [Smarty Pants](https://daringfireball.net/projects/smartypants/) in the output\n- `ignore_strings`, `postprocessor` and `registered_processors`: processors that modify the AST returned from\n\n   Earmark.Parser.as_ast/`2` before rendering (`post` because preprocessing is done on the markdown, e.g. `eex`)\n   Refer to the moduledoc of Earmark.`Transform` for details\n\nAll other options are passed onto Earmark.Parser.as_ast/`2`","ref":"Earmark.Options.html"},{"type":"function","title":"Earmark.Options.make_options/1","doc":"Make a legal and normalized Option struct from, maps or keyword lists\n\nWithout a param or an empty input we just get a new Option struct\n\niex(1)> { make_options(), make_options(%{}) }\n{ {:ok, %Earmark.Options{}}, {:ok, %Earmark.Options{}} }\n\nThe same holds for the bang version of course\n\niex(2)> { make_options!(), make_options!(%{}) }\n{ %Earmark.Options{}, %Earmark.Options{} }\n\n\nWe check for unallowed keys\n\niex(3)> make_options(no_such_option: true)\n{:error, [{:warning, 0, \"Unrecognized option no_such_option: true\"}]}\n\nOf course we do not let our users discover one error after another\n\niex(4)> make_options(no_such_option: true, gfm: false, still_not_an_option: 42)\n{:error, [{:warning, 0, \"Unrecognized option no_such_option: true\"}, {:warning, 0, \"Unrecognized option still_not_an_option: 42\"}]}\n\nAnd the bang version will raise an `Earmark.Error` as excepted (sic)\n\niex(5)> make_options!(no_such_option: true, gfm: false, still_not_an_option: 42)\n** (Earmark.Error) [{:warning, 0, \"Unrecognized option no_such_option: true\"}, {:warning, 0, \"Unrecognized option still_not_an_option: 42\"}]\n\nSome values need to be numeric\n\niex(6)> make_options(line: \"42\")\n{:error, [{:error, 0, \"line option must be numeric\"}]}\n\niex(7)> make_options(%Earmark.Options{footnote_offset: \"42\"})\n{:error, [{:error, 0, \"footnote_offset option must be numeric\"}]}\n\niex(8)> make_options(%{line: \"42\", footnote_offset: nil})\n{:error, [{:error, 0, \"footnote_offset option must be numeric\"}, {:error, 0, \"line option must be numeric\"}]}","ref":"Earmark.Options.html#make_options/1"},{"type":"function","title":"Earmark.Options.make_options!/1","doc":"","ref":"Earmark.Options.html#make_options!/1"},{"type":"function","title":"Earmark.Options.relative_filename/2","doc":"Allows to compute the path of a relative file name (starting with `\"./\"`) from the file in options\nand return an updated options struct\n\niex(9)> options = %Earmark.Options{file: \"some/path/xxx.md\"}\n...(9)> options_ = relative_filename(options, \"./local.md\")\n...(9)> options_.file\n\"some/path/local.md\"\n\nFor your convenience you can just use a keyword list\n\niex(10)> options = relative_filename([file: \"some/path/_.md\", breaks: true], \"./local.md\")\n...(10)> {options.file, options.breaks}\n{\"some/path/local.md\", true}\n\nIf the filename is not absolute it just replaces the file in options\n\niex(11)> options = %Earmark.Options{file: \"some/path/xxx.md\"}\n...(11)> options_ = relative_filename(options, \"local.md\")\n...(11)> options_.file\n\"local.md\"\n\nAnd there is a special case when processing stdin, meaning that `file: nil` we replace file\nverbatim in that case\n\niex(12)> options = %Earmark.Options{}\n...(12)> options_ = relative_filename(options, \"./local.md\")\n...(12)> options_.file\n\"./local.md\"","ref":"Earmark.Options.html#relative_filename/2"},{"type":"function","title":"Earmark.Options.with_postprocessor/2","doc":"A convenience constructor","ref":"Earmark.Options.html#with_postprocessor/2"},{"type":"module","title":"Earmark.Parser","doc":"#","ref":"Earmark.Parser.html"},{"type":"module","title":"API - Earmark.Parser","doc":"##","ref":"Earmark.Parser.html#module-api"},{"type":"module","title":"Earmark.Parser.as_ast - Earmark.Parser","doc":"This is the structure of the result of `as_ast`.\n\n    {:ok, ast, []}                   = Earmark.Parser.as_ast(markdown)\n    {:ok, ast, deprecation_messages} = Earmark.Parser.as_ast(markdown)\n    {:error, ast, error_messages}    = Earmark.Parser.as_ast(markdown)\n\nFor examples see the functiondoc below.\n\n##","ref":"Earmark.Parser.html#module-earmark-parser-as_ast"},{"type":"module","title":"Options - Earmark.Parser","doc":"Options can be passed into `as_ast/2` according to the documentation of `Earmark.Parser.Options`.\n\n    {status, ast, errors} = Earmark.Parser.as_ast(markdown, options)","ref":"Earmark.Parser.html#module-options"},{"type":"module","title":"Supports - Earmark.Parser","doc":"Standard [Gruber markdown][gruber].\n\n[gruber]:","ref":"Earmark.Parser.html#module-supports"},{"type":"module","title":"Extensions - Earmark.Parser","doc":"#","ref":"Earmark.Parser.html#module-extensions"},{"type":"module","title":"Links - Earmark.Parser","doc":"##","ref":"Earmark.Parser.html#module-links"},{"type":"module","title":"Links supported by default - Earmark.Parser","doc":"###","ref":"Earmark.Parser.html#module-links-supported-by-default"},{"type":"module","title":"Oneline HTML Link tags - Earmark.Parser","doc":"iex(1)> Earmark.Parser.as_ast(~s{ link })\n    {:ok, [{\"a\", [{\"href\", \"href\"}], [\"link\"], %{verbatim: true}}], []}\n\n###","ref":"Earmark.Parser.html#module-oneline-html-link-tags"},{"type":"module","title":"Markdown links - Earmark.Parser","doc":"New style ...\n\n    iex(2)> Earmark.Parser.as_ast(~s{[title](destination)})\n    {:ok,  [{\"p\", [], [{\"a\", [{\"href\", \"destination\"}], [\"title\"], %{}}], %{}}], []}\n\nand old style\n\n    iex(3)> Earmark.Parser.as_ast(\"[foo]: /url \\\"title\\\"\\n\\n[foo]\\n\")\n    {:ok, [{\"p\", [], [{\"a\", [{\"href\", \"/url\"}, {\"title\", \"title\"}], [\"foo\"], %{}}], %{}}], []}\n\n##","ref":"Earmark.Parser.html#module-markdown-links"},{"type":"module","title":"Autolinks - Earmark.Parser","doc":"iex(4)> Earmark.Parser.as_ast(\" \")\n    {:ok, [{\"p\", [], [{\"a\", [{\"href\", \"https://elixir-lang.com\"}], [\"https://elixir-lang.com\"], %{}}], %{}}], []}\n\n##","ref":"Earmark.Parser.html#module-autolinks"},{"type":"module","title":"Additional link parsing via options - Earmark.Parser","doc":"##","ref":"Earmark.Parser.html#module-additional-link-parsing-via-options"},{"type":"module","title":"Pure links - Earmark.Parser","doc":"**N.B.** that the `pure_links` option is `true` by default\n\n    iex(5)> Earmark.Parser.as_ast(\"https://github.com\")\n    {:ok, [{\"p\", [], [{\"a\", [{\"href\", \"https://github.com\"}], [\"https://github.com\"], %{}}], %{}}], []}\n\nBut can be deactivated\n\n    iex(6)> Earmark.Parser.as_ast(\"https://github.com\", pure_links: false)\n    {:ok, [{\"p\", [], [\"https://github.com\"], %{}}], []}\n\n\n  ##","ref":"Earmark.Parser.html#module-pure-links"},{"type":"module","title":"Wikilinks... - Earmark.Parser","doc":"are disabled by default\n\n    iex(7)> Earmark.Parser.as_ast(\"[[page]]\")\n    {:ok, [{\"p\", [], [\"[[page]]\"], %{}}], []}\n\n  and can be enabled\n\n    iex(8)> Earmark.Parser.as_ast(\"[[page]]\", wikilinks: true)\n    {:ok, [{\"p\", [], [{\"a\", [{\"href\", \"page\"}], [\"page\"], %{wikilink: true}}], %{}}], []}\n\n\n#","ref":"Earmark.Parser.html#module-wikilinks"},{"type":"module","title":"Sub and Sup HTML Elements - Earmark.Parser","doc":"This feature is not enabled by default but can be enabled with the option `sub_sup: true`\n\nTherefore we will get\n\n    iex(9)> Earmark.Parser.as_ast(\"H~2~O or a^n^ + b^n^ = c^n^\")\n    {:ok, [{\"p\", [], [\"H~2~O or a^n^ + b^n^ = c^n^\"], %{}}], []}\n\nBut by specifying `sub_sup: true`\n\n    iex(10)> Earmark.Parser.as_ast(\"H~2~O or a^n^ + b^n^ = c^n^\", sub_sup: true)\n    {:ok, [{\"p\", [], [\"H\", {\"sub\", [], [\"2\"], %{}}, \"O or a\", {\"sup\", [], [\"n\"], %{}}, \" + b\", {\"sup\", [], [\"n\"], %{}}, \" = c\", {\"sup\", [], [\"n\"], %{}}], %{}}], []}\n\n#","ref":"Earmark.Parser.html#module-sub-and-sup-html-elements"},{"type":"module","title":"Github Flavored Markdown - Earmark.Parser","doc":"GFM is supported by default, however as GFM is a moving target and all GFM extension do not make sense in a general context, Earmark.Parser does not support all of it, here is a list of what is supported:\n\n##","ref":"Earmark.Parser.html#module-github-flavored-markdown"},{"type":"module","title":"Strike Through - Earmark.Parser","doc":"iex(11)> Earmark.Parser.as_ast(\"~~hello~~\")\n    {:ok, [{\"p\", [], [{\"del\", [], [\"hello\"], %{}}], %{}}], []}\n\n##","ref":"Earmark.Parser.html#module-strike-through"},{"type":"module","title":"GFM Tables - Earmark.Parser","doc":"Are not enabled by default\n\n    iex(12)> as_ast(\"a|b\\\\n-|-\\\\nc|d\\\\n\")\n    {:ok, [{\"p\", [], [\"a|b\\\\n-|-\\\\nc|d\\\\n\"], %{}}], []}\n\nBut can be enabled with `gfm_tables: true`\n\n    iex(13)> as_ast(\"a|b\\n-|-\\nc|d\\n\", gfm_tables: true)\n    {:ok,\n      [\n        {\n          \"table\",\n          [],\n          [\n            {\"thead\", [], [{\"tr\", [], [{\"th\", [{\"style\", \"text-align: left;\"}], [\"a\"], %{}}, {\"th\", [{\"style\", \"text-align: left;\"}], [\"b\"], %{}}], %{}}], %{}},\n            {\"tbody\", [], [{\"tr\", [], [{\"td\", [{\"style\", \"text-align: left;\"}], [\"c\"], %{}}, {\"td\", [{\"style\", \"text-align: left;\"}], [\"d\"], %{}}], %{}}], %{}}\n          ],\n          %{}\n        }\n      ],\n      []}\n\n##","ref":"Earmark.Parser.html#module-gfm-tables"},{"type":"module","title":"Syntax Highlighting - Earmark.Parser","doc":"All backquoted or fenced code blocks with a language string are rendered with the given\nlanguage as a _class_ attribute of the _code_ tag.\n\nFor example:\n\n    iex(14)> [\n    ...(14)>    \"```elixir\",\n    ...(14)>    \" @tag :hello\",\n    ...(14)>    \"```\"\n    ...(14)> ] |> as_ast()\n    {:ok, [{\"pre\", [], [{\"code\", [{\"class\", \"elixir\"}], [\" @tag :hello\"], %{}}], %{}}], []}\n\nwill be rendered as shown in the doctest above.\n\nIf you want to integrate with a syntax highlighter with different conventions you can add more classes by specifying prefixes that will be\nput before the language string.\n\nPrism.js for example needs a class `language-elixir`. In order to achieve that goal you can add `language-`\nas a `code_class_prefix` to `Earmark.Parser.Options`.\n\nIn the following example we want more than one additional class, so we add more prefixes.\n\n    iex(15)> [\n    ...(15)>    \"```elixir\",\n    ...(15)>    \" @tag :hello\",\n    ...(15)>    \"```\"\n    ...(15)> ] |> as_ast(%Earmark.Parser.Options{code_class_prefix: \"lang- language-\"})\n    {:ok, [{\"pre\", [], [{\"code\", [{\"class\", \"elixir lang-elixir language-elixir\"}], [\" @tag :hello\"], %{}}], %{}}], []}\n\n\n##","ref":"Earmark.Parser.html#module-syntax-highlighting"},{"type":"module","title":"Footnotes - Earmark.Parser","doc":"**N.B.** Footnotes are disabled by default, use `as_ast(..., footnotes: true)` to enable them\n\nFootnotes are now a **superset** of GFM Footnotes. This implies some changes\n\n  - Footnote definitions (`[^footnote_id]`) must come at the end of your document (_GFM_)\n  - Footnotes that are not referenced are not rendered anymore (_GFM_)\n  - Footnote definitions can contain any markup with the exception of footnote definitions\n\n        # iex(16)> markdown = [\n        # ...(16)> \"My reference[^to_footnote]\",\n        # ...(16)> \"\",\n        # ...(16)> \"[^1]: I am not rendered\",\n        # ...(16)> \"[^to_footnote]: Important information\"]\n        # ...(16)> {:ok, ast, []} = as_ast(markdown, footnotes: true)\n        # ...(16)> ast\n        # [\n        #   {\"p\", [], [\"My reference\",\n        #     {\"a\",\n        #     [{\"href\", \"#fn:to_footnote\"}, {\"id\", \"fnref:to_footnote\"}, {\"class\", \"footnote\"}, {\"title\", \"see footnote\"}],\n        #     [\"to_footnote\"], %{}}\n        #   ], %{}},\n        #   {\"div\",\n        #   [{\"class\", \"footnotes\"}],\n        #   [{\"hr\", [], [], %{}},\n        #     {\"ol\", [],\n        #     [{\"li\", [{\"id\", \"fn:to_footnote\"}],\n        #       [{\"a\", [{\"title\", \"return to article\"}, {\"class\", \"reversefootnote\"}, {\"href\", \"#fnref:to_footnote\"}], [\"&#x21A9;\"], %{}},\n        #         {\"p\", [], [\"Important information\"], %{}}], %{}}\n        #     ], %{}}], %{}}\n        # ]\n\n  For more complex examples of footnotes, please refer to\n  [these tests](https://github.com/RobertDober/earmark_parser/tree/master/test/acceptance/ast/footnotes/multiple_fn_test.exs)\n\n##","ref":"Earmark.Parser.html#module-footnotes"},{"type":"module","title":"Breaks - Earmark.Parser","doc":"Hard linebreaks are disabled by default\n\n        iex(17)> [\"* a\",\"  b\", \"c\"]\n        ...(17)> |> as_ast()\n        {:ok,\n          [{\"ul\", [], [{\"li\", [], [\"a\\nb\\nc\"], %{}}], %{}}],\n          []}\n\n    But can be enabled with `breaks: true`\n\n        iex(18)> [\"* a\",\"  b\", \"c\"]\n        ...(18)> |> as_ast(breaks: true)\n        {:ok, [{\"ul\", [], [{\"li\", [], [\"a\", {\"br\", [], [], %{}}, \"b\", {\"br\", [], [], %{}}, \"c\"], %{}}], %{}}], []}\n\n##","ref":"Earmark.Parser.html#module-breaks"},{"type":"module","title":"Enabling **all** options that are disabled by default - Earmark.Parser","doc":"Can be achieved with the `all: true` option\n\n        iex(19)> [\n        ...(19)> \"a^n^\",\n        ...(19)> \"b~2~\",\n        ...(19)> \"[[wikilink]]\"]\n        ...(19)> |> as_ast(all: true)\n        {:ok, [\n          {\"p\", [], [\"a\", {\"sup\", [], [\"n\"], %{}}, {\"br\", [], [], %{}}, \"b\", {\"sub\", [], [\"2\"], %{}}, {\"br\", [], [], %{}}, {\"a\", [{\"href\", \"wikilink\"}], [\"wikilink\"], %{wikilink: true}}], %{}}\n          ],\n          []}\n\n##","ref":"Earmark.Parser.html#module-enabling-all-options-that-are-disabled-by-default"},{"type":"module","title":"Tables - Earmark.Parser","doc":"Are supported as long as they are preceded by an empty line.\n\n    State | Abbrev | Capital\n    ----: | :----: | -------\n    Texas | TX     | Austin\n    Maine | ME     | Augusta\n\nTables may have leading and trailing vertical bars on each line\n\n    | State | Abbrev | Capital |\n    | ----: | :----: | ------- |\n    | Texas | TX     | Austin  |\n    | Maine | ME     | Augusta |\n\nTables need not have headers, in which case all column alignments\ndefault to left.\n\n    | Texas | TX     | Austin  |\n    | Maine | ME     | Augusta |\n\nCurrently we assume there are always spaces around interior vertical unless\nthere are exterior bars.\n\nHowever in order to be more GFM compatible the `gfm_tables: true` option\ncan be used to interpret only interior vertical bars as a table if a separation\nline is given, therefore\n\n     Language|Rating\n     --------|------\n     Elixir  | awesome\n\nis a table (if and only if `gfm_tables: true`) while\n\n     Language|Rating\n     Elixir  | awesome\n\nnever is.\n\n##","ref":"Earmark.Parser.html#module-tables"},{"type":"module","title":"HTML Blocks - Earmark.Parser","doc":"HTML is not parsed recursively or detected in all conditions right now, though GFM compliance\nis a goal.\n\nBut for now the following holds:\n\nA HTML Block defined by a tag starting a line and the same tag starting a different line is parsed\nas one HTML AST node, marked with %{verbatim: true}\n\nE.g.\n\n    iex(20)> lines = [ \"  \", \"some  \", \" more text\" ]\n    ...(20)> Earmark.Parser.as_ast(lines)\n    {:ok, [{\"div\", [], [\" \", \"some  \"], %{verbatim: true}}, \"more text\"], []}\n\nAnd a line starting with an opening tag and ending with the corresponding closing tag is parsed in similar\nfashion\n\n    iex(21)> Earmark.Parser.as_ast([\" spaniel \"])\n    {:ok, [{\"span\", [{\"class\", \"superspan\"}], [\"spaniel\"], %{verbatim: true}}], []}\n\nWhat is HTML?\n\nWe differ from strict GFM by allowing **all** tags not only HTML5 tags this holds for one liners....\n\n    iex(22)> {:ok, ast, []} = Earmark.Parser.as_ast([\" \", \" better \"])\n    ...(22)> ast\n    [\n      {\"stupid\", [], [], %{verbatim: true}},\n      {\"not\", [], [\"better\"], %{verbatim: true}}]\n\nand for multi line blocks\n\n    iex(23)> {:ok, ast, []} = Earmark.Parser.as_ast([ \" \", \"world\", \" \"])\n    ...(23)> ast\n    [{\"hello\", [], [\"world\"], %{verbatim: true}}]\n\n##","ref":"Earmark.Parser.html#module-html-blocks"},{"type":"module","title":"HTML Comments - Earmark.Parser","doc":"Are recognized if they start a line (after ws and are parsed until the next `-->` is found\nall text after the next '-->' is ignored\n\nE.g.\n\n    iex(24)> Earmark.Parser.as_ast(\"   text -->\\nafter\")\n    {:ok, [{:comment, [], [\" Comment\", \"comment line\", \"comment \"], %{comment: true}}, {\"p\", [], [\"after\"], %{}}], []}\n\n\n##","ref":"Earmark.Parser.html#module-html-comments"},{"type":"module","title":"Lists - Earmark.Parser","doc":"Lists are pretty much GFM compliant, but some behaviors concerning the interpreation of the markdown inside a List Item's first\nparagraph seem not worth to be interpreted, examples are blockquote in a tight [list item](ttps://babelmark.github.io/?text=*+aa%0A++%3E+Second)\nwhich we can only have in a [loose one](https://babelmark.github.io/?text=*+aa%0A++%0A++%3E+Second)\n\nOr a headline in a [tight list item](https://babelmark.github.io/?text=*+bb%0A++%23+Headline) which, again is only available in the\n[loose version](https://babelmark.github.io/?text=*+bb%0A%0A++%23+Headline) in Earmark.Parser.\n\nfurthermore [this example](https://babelmark.github.io/?text=*+aa%0A++%60%60%60%0ASecond%0A++%60%60%60) demonstrates how weird\nand definitely not useful GFM's own interpretation can get.\n\nTherefore we stick to a more predictable approach.\n\n      iex(25)> markdown = [\n      ...(25)> \"* aa\",\n      ...(25)> \"  ```\",\n      ...(25)> \"Second\",\n      ...(25)> \"  ```\" ]\n      ...(25)> as_ast(markdown)\n      {:ok, [{\"ul\", [], [{\"li\", [], [\"aa\", {\"pre\", [], [{\"code\", [], [\"Second\"], %{}}], %{}}], %{}}], %{}}], []}\n\nAlso we do support the immediate style of block content inside lists\n\n      iex(26)> as_ast(\"* > Nota Bene!\")\n      {:ok, [{\"ul\", [], [{\"li\", [], [{\"blockquote\", [], [{\"p\", [], [\"Nota Bene!\"], %{}}], %{}}], %{}}], %{}}], []}\n\nor\n\n      iex(27)> as_ast(\"1. # Breaking...\")\n      {:ok, [{\"ol\", [], [{\"li\", [], [{\"h1\", [], [\"Breaking...\"], %{}}], %{}}], %{}}], []}\n\n\n#","ref":"Earmark.Parser.html#module-lists"},{"type":"module","title":"Adding Attributes with the IAL extension - Earmark.Parser","doc":"##","ref":"Earmark.Parser.html#module-adding-attributes-with-the-ial-extension"},{"type":"module","title":"To block elements - Earmark.Parser","doc":"HTML attributes can be added to any block-level element. We use\nthe Kramdown syntax: add the line `{:` _attrs_ `}` following the block.\n\n    iex(28)> markdown = [\"# Headline\", \"{:.from-next-line}\"]\n    ...(28)> as_ast(markdown)\n    {:ok, [{\"h1\", [{\"class\", \"from-next-line\"}], [\"Headline\"], %{}}], []}\n\nHeaders can also have the IAL string at the end of the line\n\n    iex(29)> markdown = [\"# Headline{:.from-same-line}\"]\n    ...(29)> as_ast(markdown)\n    {:ok, [{\"h1\", [{\"class\", \"from-same-line\"}], [\"Headline\"], %{}}], []}\n\nA special use case is headers inside blockquotes which allow for some nifty styling in `ex_doc`*\nsee [this PR](https://github.com/elixir-lang/ex_doc/pull/1400) if you are interested in the technical\ndetails\n\n    iex(30)> markdown = [\"> # Headline{:.warning}\"]\n    ...(30)> as_ast(markdown)\n    {:ok, [{\"blockquote\", [], [{\"h1\", [{\"class\", \"warning\"}], [\"Headline\"], %{}}], %{}}], []}\n\nThis also works for headers inside lists\n\n    iex(31)> markdown = [\"- # Headline{:.warning}\"]\n    ...(31)> as_ast(markdown)\n    {:ok, [{\"ul\", [], [{\"li\", [], [{\"h1\", [{\"class\", \"warning\"}], [\"Headline\"], %{}}], %{}}], %{}}], []}\n\nIt still works for inline code, as it did before\n\n    iex(32)> markdown = \"`Enum.map`{:lang=elixir}\"\n    ...(32)> as_ast(markdown)\n    {:ok, [{\"p\", [], [{\"code\", [{\"class\", \"inline\"}, {\"lang\", \"elixir\"}], [\"Enum.map\"], %{}}], %{}}], []}\n\n\n_attrs_ can be one or more of:\n\n  * `.className`\n  * `#id`\n  * name=value, name=\"value\", or name='value'\n\nFor example:\n\n    # Warning\n    {: .red}\n\n    Do not turn off the engine\n    if you are at altitude.\n    {: .boxed #warning spellcheck=\"true\"}\n\n##","ref":"Earmark.Parser.html#module-to-block-elements"},{"type":"module","title":"To links or images - Earmark.Parser","doc":"It is possible to add IAL attributes to generated links or images in the following\nformat.\n\n    iex(33)> markdown = \"[link](url) {: .classy}\"\n    ...(33)> Earmark.Parser.as_ast(markdown)\n    { :ok, [{\"p\", [], [{\"a\", [{\"class\", \"classy\"}, {\"href\", \"url\"}], [\"link\"], %{}}], %{}}], []}\n\nFor both cases, malformed attributes are ignored and warnings are issued.\n\n    iex(34)> [ \"Some text\", \"{:hello}\" ] |> Enum.join(\"\\n\") |> Earmark.Parser.as_ast()\n    {:error, [{\"p\", [], [\"Some text\"], %{}}], [{:warning, 2,\"Illegal attributes [\\\"hello\\\"] ignored in IAL\"}]}\n\nIt is possible to escape the IAL in both forms if necessary\n\n    iex(35)> markdown = \"[link](url)\\\\{: .classy}\"\n    ...(35)> Earmark.Parser.as_ast(markdown)\n    {:ok, [{\"p\", [], [{\"a\", [{\"href\", \"url\"}], [\"link\"], %{}}, \"{: .classy}\"], %{}}], []}\n\nThis of course is not necessary in code blocks or text lines\ncontaining an IAL-like string, as in the following example\n\n    iex(36)> markdown = \"hello {:world}\"\n    ...(36)> Earmark.Parser.as_ast(markdown)\n    {:ok, [{\"p\", [], [\"hello {:world}\"], %{}}], []}","ref":"Earmark.Parser.html#module-to-links-or-images"},{"type":"module","title":"Limitations - Earmark.Parser","doc":"* Block-level HTML is correctly handled only if each HTML\n    tag appears on its own line. So\n\n         \n         \n        hello\n         \n         \n\n    will work. However. the following won't\n\n         \n        hello \n\n  * John Gruber's tests contain an ambiguity when it comes to\n    lines that might be the start of a list inside paragraphs.\n\n    One test says that\n\n        This is the text\n        * of a paragraph\n        that I wrote\n\n    is a single paragraph. The \"*\" is not significant. However, another\n    test has\n\n        *   A list item\n            * an another\n\n    and expects this to be a nested list. But, in reality, the second could just\n    be the continuation of a paragraph.\n\n    I've chosen always to use the second interpretation—a line that looks like\n    a list item will always be a list item.\n\n  * Rendering of block and inline elements.\n\n    Block or void HTML elements that are at the absolute beginning of a line end\n    the preceding paragraph.\n\n    Thusly\n\n        mypara\n         \n\n    Becomes\n\n         mypara \n         \n\n    While\n\n        mypara\n          \n\n    will be transformed into\n\n         mypara","ref":"Earmark.Parser.html#module-limitations"},{"type":"module","title":"Annotations - Earmark.Parser","doc":"**N.B.** this is an experimental feature from v1.4.16-pre on and might change or be removed again\n\nThe idea is that each markdown line can be annotated, as such annotations change the semantics of Markdown\nthey have to be enabled with the `annotations` option.\n\nIf the `annotations` option is set to a string (only one string is supported right now, but a list might\nbe implemented later on, hence the name), the last occurrence of that string in a line and all text following\nit will be added to the line as an annotation.\n\nDepending on how that line will eventually be parsed, this annotation will be added to the meta map (the 4th element\nin an AST quadruple) with the key `:annotation`\n\nIn the current version the annotation will only be applied to verbatim HTML tags and paragraphs\n\nLet us show some examples now:\n\n#","ref":"Earmark.Parser.html#module-annotations"},{"type":"module","title":"Annotated Paragraphs - Earmark.Parser","doc":"iex(37)> as_ast(\"hello %> annotated\", annotations: \"%>\")\n    {:ok, [{\"p\", [], [\"hello \"], %{annotation: \"%> annotated\"}}], []}\n\nIf we annotate more than one line in a para the first annotation takes precedence\n\n    iex(38)> as_ast(\"hello %> annotated\\nworld %> discarded\", annotations: \"%>\")\n    {:ok, [{\"p\", [], [\"hello \\nworld \"], %{annotation: \"%> annotated\"}}], []}\n\n#","ref":"Earmark.Parser.html#module-annotated-paragraphs"},{"type":"module","title":"Annotated HTML elements - Earmark.Parser","doc":"In one line\n\n    iex(39)> as_ast(\" One Line  // a span\", annotations: \"//\")\n    {:ok, [{\"span\", [], [\"One Line\"], %{annotation: \"// a span\", verbatim: true}}], []}\n\nor block elements\n\n    iex(40)> [\n    ...(40)> \"  : annotation\",\n    ...(40)> \"   text \",\n    ...(40)> \"  : discarded\"\n    ...(40)> ] |> as_ast(annotations: \" : \")\n    {:ok, [{\"div\", [], [\"   text \"], %{annotation: \" : annotation\", verbatim: true}}], []}\n\n#","ref":"Earmark.Parser.html#module-annotated-html-elements"},{"type":"module","title":"Commenting your Markdown - Earmark.Parser","doc":"Although many markdown elements do not support annotations yet, they can be used to comment your markdown, w/o cluttering\nthe generated AST with comments\n\n    iex(41)> [\n    ...(41)> \"# Headline --> first line\",\n    ...(41)> \"- item1 --> a list item\",\n    ...(41)> \"- item2 --> another list item\",\n    ...(41)> \"\",\n    ...(41)> \"  --> do not go there\"\n    ...(41)> ] |> as_ast(annotations: \"-->\")\n    {:ok, [\n      {\"h1\", [], [\"Headline\"], %{}},\n      {\"ul\", [], [{\"li\", [], [\"item1 \"], %{}}, {\"li\", [], [\"item2 \"], %{}}], %{}},\n      {\"p\", [], [{\"a\", [{\"href\", \"http://somewhere/to/go\"}], [\"http://somewhere/to/go\"], %{}}, \" \"], %{annotation: \"--> do not go there\"}}\n      ], []\n     }","ref":"Earmark.Parser.html#module-commenting-your-markdown"},{"type":"function","title":"Earmark.Parser.as_ast/2","doc":"iex(42)> markdown = \"My `code` is **best**\"\n    ...(42)> {:ok, ast, []} = Earmark.Parser.as_ast(markdown)\n    ...(42)> ast\n    [{\"p\", [], [\"My \", {\"code\", [{\"class\", \"inline\"}], [\"code\"], %{}}, \" is \", {\"strong\", [], [\"best\"], %{}}], %{}}]\n\n\n\n    iex(43)> markdown = \"```elixir\\nIO.puts 42\\n```\"\n    ...(43)> {:ok, ast, []} = Earmark.Parser.as_ast(markdown, code_class_prefix: \"lang-\")\n    ...(43)> ast\n    [{\"pre\", [], [{\"code\", [{\"class\", \"elixir lang-elixir\"}], [\"IO.puts 42\"], %{}}], %{}}]\n\n**Rationale**:\n\nThe AST is exposed in the spirit of [Floki's](https://hex.pm/packages/floki).","ref":"Earmark.Parser.html#as_ast/2"},{"type":"function","title":"Earmark.Parser.version/0","doc":"Accesses current hex version of the `Earmark.Parser` application. Convenience for\n  `iex` usage.","ref":"Earmark.Parser.html#version/0"},{"type":"type","title":"Earmark.Parser.ast/0","doc":"","ref":"Earmark.Parser.html#t:ast/0"},{"type":"type","title":"Earmark.Parser.ast_attribute/0","doc":"","ref":"Earmark.Parser.html#t:ast_attribute/0"},{"type":"type","title":"Earmark.Parser.ast_attribute_name/0","doc":"","ref":"Earmark.Parser.html#t:ast_attribute_name/0"},{"type":"type","title":"Earmark.Parser.ast_attribute_value/0","doc":"","ref":"Earmark.Parser.html#t:ast_attribute_value/0"},{"type":"type","title":"Earmark.Parser.ast_attributes/0","doc":"","ref":"Earmark.Parser.html#t:ast_attributes/0"},{"type":"type","title":"Earmark.Parser.ast_meta/0","doc":"","ref":"Earmark.Parser.html#t:ast_meta/0"},{"type":"type","title":"Earmark.Parser.ast_node/0","doc":"","ref":"Earmark.Parser.html#t:ast_node/0"},{"type":"type","title":"Earmark.Parser.ast_tag/0","doc":"","ref":"Earmark.Parser.html#t:ast_tag/0"},{"type":"type","title":"Earmark.Parser.ast_tuple/0","doc":"","ref":"Earmark.Parser.html#t:ast_tuple/0"},{"type":"module","title":"Earmark.Parser.Enum.Ext","doc":"Some extensions of Enum functions","ref":"Earmark.Parser.Enum.Ext.html"},{"type":"function","title":"Earmark.Parser.Enum.Ext.reduce_with_end/3","doc":"`reduce_with_end` is like `Enum.reduce` for lists, but the reducer function is called for\neach element of the list with the tuple `{:element, element}` and the accumulator and once\nmore at the end with `:end` and the accumulator\n\n    iex(1)> reducer =\n    ...(1)>   fn {:element, nil}, {partial, result} -> {[], [Enum.sum(partial)|result]}\n    ...(1)>      {:element, val}, {partial, result} -> {[val|partial], result}\n    ...(1)>      :end,            {partial, result} -> [Enum.sum(partial)|result] |> Enum.reverse\n    ...(1)>   end\n    ...(1)> [1, 2, nil, 4, 1, 0, nil, 3, 2, 2]\n    ...(1)> |> reduce_with_end({[], []}, reducer)\n    [3, 5, 7]\n\n**N.B.** that in the treatment of `:end` we can change the shape of the accumulator w/o any\npenalty concerning the complexity of the reducer function","ref":"Earmark.Parser.Enum.Ext.html#reduce_with_end/3"},{"type":"function","title":"Earmark.Parser.Enum.Ext.reverse_map_reduce/3","doc":"Like map_reduce but reversing the list\n\n    iex(2)> replace_nil_and_count = fn ele, acc ->\n    ...(2)>   if ele, do: {ele, acc}, else: {\"\", acc + 1}\n    ...(2)> end\n    ...(2)> [\"y\", nil, \"u\", nil, nil, \"a\", nil] |> reverse_map_reduce(0, replace_nil_and_count)\n    { [\"\", \"a\", \"\", \"\", \"u\", \"\", \"y\"], 4 }","ref":"Earmark.Parser.Enum.Ext.html#reverse_map_reduce/3"},{"type":"module","title":"Earmark.Parser.Options","doc":"","ref":"Earmark.Parser.Options.html"},{"type":"function","title":"Earmark.Parser.Options.normalize/1","doc":"Use normalize before passing it into any API function\n\n      iex(1)> options = normalize(annotations: \"%%\")\n      ...(1)> options.annotations\n      ~r{\\A(.*)(%%.*)}","ref":"Earmark.Parser.Options.html#normalize/1"},{"type":"module","title":"Earmark.Restructure","doc":"","ref":"Earmark.Restructure.html"},{"type":"function","title":"Earmark.Restructure.merge_lists/3","doc":"Given two lists that are either of equal length, or with the first list\nexactly one element longer than the second, returns a list that begins with\nthe first element from the first list, then the first element from the second\nlist, and so forth until both lists are empty.","ref":"Earmark.Restructure.html#merge_lists/3"},{"type":"function","title":"Earmark.Restructure.split_by_regex/3","doc":"Utility for creating a restructuring that parses text by splitting it into\nparts \"of interest\" vs. \"other parts\" using a regular expression.\nReturns a list of parts where the parts matching regex have been processed\nby invoking map_captures_fn on each part, and a list of remaining parts,\npreserving the order of parts from what it was in the plain text item.\n\n      iex(2)> input = \"This is ::all caps::, right?\"\n      ...(2)> split_by_regex(input, ~r/::(.*?)::/, fn [_, inner|_] -> String.upcase(inner) end)\n      [\"This is \", \"ALL CAPS\", \", right?\"]","ref":"Earmark.Restructure.html#split_by_regex/3"},{"type":"function","title":"Earmark.Restructure.walk_and_modify_ast/4","doc":"Walks an AST and allows you to process it (storing details in acc) and/or\nmodify it as it is walked.\n\nitems is the AST you got from Earmark.Parser.as_ast()\n\nacc is the initial value of an accumulator that is passed to both\nprocess_item_fn and process_list_fn and accumulated. If your functions\ndo not need to use or store any state, you can pass nil.\n\nThe process_item_fn function is required. It takes two parameters, the\nsingle item to process (which will either be a string or a 4-tuple) and\nthe accumulator, and returns a tuple {processed_item, updated_acc}.\nReturning the empty list for processed_item will remove the item processed\nthe AST.\n\nThe process_list_fn function is optional and defaults to no modification of\nitems or accumulator. It takes two parameters, the list of items that\nare the sub-items of a given element in the AST (or the top-level list of\nitems), and the accumulator, and returns a tuple\n{processed_items_list, updated_acc}.\n\nThis function ends up returning {ast, acc}.\n\nHere is an example using a custom format to make ` ` nodes and allowing\ncommented text to be left out\n\n    iex(1)> is_comment? = fn item -> is_binary(item) && Regex.match?(~r/\\A\\s*--/, item) end\n    ...(1)> comment_remover =\n    ...(1)>   fn items, acc -> {Enum.reject(items, is_comment?), acc} end\n    ...(1)> italics_maker = fn\n    ...(1)>   item, acc when is_binary(item) ->\n    ...(1)>     new_item = Restructure.split_by_regex(\n    ...(1)>       item,\n    ...(1)>       ~r/\\/([[:graph:]].*?[[:graph:]]|[[:graph:]])\\//,\n    ...(1)>       fn [_, content] ->\n    ...(1)>         {\"em\", [], [content], %{}}\n    ...(1)>       end\n    ...(1)>     )\n    ...(1)>     {new_item, acc}\n    ...(1)>   item, \"a\" -> {item, nil}\n    ...(1)>   {name, _, _, _}=item, _ -> {item, name}\n    ...(1)> end\n    ...(1)> markdown = \"\"\"\n    ...(1)> [no italics in links](http://example.io/some/path)\n    ...(1)> but /here/\n    ...(1)>\n    ...(1)> -- ignore me\n    ...(1)>\n    ...(1)> text\n    ...(1)> \"\"\"\n    ...(1)> {:ok, ast, []} = Earmark.Parser.as_ast(markdown)\n    ...(1)> Restructure.walk_and_modify_ast(ast, nil, italics_maker, comment_remover)\n    {[\n      {\"p\", [],\n        [\n          {\"a\", [{\"href\", \"http://example.io/some/path\"}], [\"no italics in links\"],\n          %{}},\n          \"\\nbut \",\n          {\"em\", [], [\"here\"], %{}},\n          \"\"\n        ], %{}},\n        {\"p\", [], [], %{}},\n        {\"p\", [], [\"text\"], %{}}\n      ], \"p\"}","ref":"Earmark.Restructure.html#walk_and_modify_ast/4"},{"type":"module","title":"Earmark.SysInterface","doc":"","ref":"Earmark.SysInterface.html"},{"type":"function","title":"Earmark.SysInterface.readlines/1","doc":"A proxy to IO.stream(..., :line) or usage with a filename","ref":"Earmark.SysInterface.html#readlines/1"},{"type":"module","title":"Earmark.TagSpecificProcessors","doc":"This struct represents a list of tuples `{tag, function}` from which a postprocessing function\ncan be constructed\n\nGeneral Usage Examples:\n\n    iex(0)> tsp = new({\"p\", &Earmark.AstTools.merge_atts_in_node(&1, class: \"one\")})\n    ...(0)> tsp = prepend_tag_function(tsp, \"i\", &Earmark.AstTools.merge_atts_in_node(&1, class: \"two\"))\n    ...(0)> make_postprocessor(tsp).({\"p\", [], nil, nil})\n    {\"p\", [{\"class\", \"one\"}], nil, nil}\n\n    iex(1)> tsp = new({\"p\", &Earmark.AstTools.merge_atts_in_node(&1, class: \"one\")})\n    ...(1)> tsp = prepend_tag_function(tsp, \"i\", &Earmark.AstTools.merge_atts_in_node(&1, class: \"two\"))\n    ...(1)> make_postprocessor(tsp).({\"i\", [{\"class\", \"i\"}], nil, nil})\n    {\"i\", [{\"class\", \"two i\"}], nil, nil}\n\n    iex(2)> tsp = new({\"p\", &Earmark.AstTools.merge_atts_in_node(&1, class: \"one\")})\n    ...(2)> tsp = prepend_tag_function(tsp, \"i\", &Earmark.AstTools.merge_atts_in_node(&1, class: \"two\"))\n    ...(2)> make_postprocessor(tsp).({\"x\", [], nil, nil})\n    {\"x\", [], nil, nil}","ref":"Earmark.TagSpecificProcessors.html"},{"type":"function","title":"Earmark.TagSpecificProcessors.make_postprocessor/1","doc":"Constructs a postprocessor function from this struct which will find the function associated\nto the tag of the node, and apply the node to it if such a function was found.","ref":"Earmark.TagSpecificProcessors.html#make_postprocessor/1"},{"type":"function","title":"Earmark.TagSpecificProcessors.new/0","doc":"Convenience construction\n\n    iex(3)> new()\n    %Earmark.TagSpecificProcessors{}","ref":"Earmark.TagSpecificProcessors.html#new/0"},{"type":"function","title":"Earmark.TagSpecificProcessors.new/1","doc":"","ref":"Earmark.TagSpecificProcessors.html#new/1"},{"type":"function","title":"Earmark.TagSpecificProcessors.prepend_tag_function/2","doc":"","ref":"Earmark.TagSpecificProcessors.html#prepend_tag_function/2"},{"type":"function","title":"Earmark.TagSpecificProcessors.prepend_tag_function/3","doc":"Prepends a tuple {tag, function} to the list of such tuples.","ref":"Earmark.TagSpecificProcessors.html#prepend_tag_function/3"},{"type":"module","title":"Earmark.Transform","doc":"##","ref":"Earmark.Transform.html"},{"type":"module","title":"Structure Conserving Transformers - Earmark.Transform","doc":"For the convenience of processing the output of `Earmark.Parser.as_ast` we expose two structure conserving\nmappers.\n\n##### `map_ast`\n\nTraverses an AST using a mapper function.\n\nThe mapper function will be called for each node including text elements unless `map_ast` is called with\nthe third positional parameter `ignore_strings`, which is optional and defaults to `false`, set to `true`.\n\nDepending on the return value of the mapper function the traversal will either\n\n- `{new_tag, new_atts, ignored, new_meta}`\n\n  just replace the `tag`, `attribute` and `meta` values of the current node with the values of the returned\n  quadruple (ignoring `ignored` for facilitating nodes w/o transformation)\n  and then descend into the **original** content of the node.\n\n\n- `{:replace, node}`\n\n  replaces the current node with `node` and does not descend anymore, but continues traversal on sibblings.\n\n\n- {new_function, {new_tag, new_atts, ignored, new_meta}}\n\n  just replace the `tag`, `attribute` and `meta` values of the current node with the values of the returned\n  quadruple (ignoring `ignored` for facilitating nodes w/o transformation)\n  and then descend into the **original** content of the node but with the mapper function `new_function`\n  used for transformation of the AST.\n\n  **N.B.** The original mapper function will be used for transforming the sibbling nodes though.\n\ntakes a function that will be called for each node of the AST, where a leaf node is either a quadruple\nlike `{\"code\", [{\"class\", \"inline\"}], [\"some code\"], %{}}` or a text leaf like `\"some code\"`\n\nThe result of the function call must be\n\n- for nodes → as described above\n\n- for strings → strings or nodes\n\nAs an example let us transform an ast to have symbol keys\n\n      iex(1)> input = [\n      ...(1)> {\"h1\", [], [\"Hello\"], %{title: true}},\n      ...(1)> {\"ul\", [], [{\"li\", [], [\"alpha\"], %{}}, {\"li\", [], [\"beta\"], %{}}], %{}}]\n      ...(1)> map_ast(input, fn {t, a, _, m} -> {String.to_atom(t), a, nil, m} end, true)\n      [ {:h1, [], [\"Hello\"], %{title: true}},\n        {:ul, [], [{:li, [], [\"alpha\"], %{}}, {:li, [], [\"beta\"], %{}}], %{}} ]\n\n**N.B.** If this returning convention is not respected `map_ast` might not complain, but the resulting\ntransformation might not be suitable for `Earmark.Transform.transform` anymore. From this follows that\nany function passed in as value of the `postprocessor:` option must obey to these conventions.\n\n##### `map_ast_with`\n\nthis is like `map_ast` but like a reducer an accumulator can also be passed through.\n\nFor that reason the function is called with two arguments, the first element being the same value\nas in `map_ast` and the second the accumulator. The return values need to be equally augmented\ntuples.\n\nA simple example, annotating traversal order in the meta map's `:count` key, as we are not\ninterested in text nodes we use the fourth parameter `ignore_strings` which defaults to `false`\n\n       iex(2)>  input = [\n       ...(2)>  {\"ul\", [], [{\"li\", [], [\"one\"], %{}}, {\"li\", [], [\"two\"], %{}}], %{}},\n       ...(2)>  {\"p\", [], [\"hello\"], %{}}]\n       ...(2)>  counter = fn {t, a, _, m}, c -> {{t, a, nil, Map.put(m, :count, c)}, c+1} end\n       ...(2)>  map_ast_with(input, 0, counter, true)\n       {[ {\"ul\", [], [{\"li\", [], [\"one\"], %{count: 1}}, {\"li\", [], [\"two\"], %{count: 2}}], %{count: 0}},\n         {\"p\", [], [\"hello\"], %{count: 3}}], 4}\n\nLet us describe an implementation of a real world use case taken from [Elixir Forum](https://elixirforum.com/t/how-to-extend-earmark/47406)\n\nSimplifying the exact parsing of the text node in this example we only want to replace a text node of the form `#elixir` with\na link to the Elixir home page _but_ only when inside a `{\"p\",....}` node\n\nWe can achieve this as follows\n\n      iex(3)> elixir_home = {\"a\", [{\"href\", \"https://elixir-lang.org\"}], [\"Elixir\"], %{}}\n      ...(3)> transformer = fn {\"p\", atts, _, meta}, _ -> {{\"p\", atts, nil, meta}, true}\n      ...(3)>                  \"#elixir\", true -> {elixir_home, false}\n      ...(3)>                  text, _ when is_binary(text) -> {text, false}\n      ...(3)>                  node, _ ->  {node, false} end\n      ...(3)> ast = [\n      ...(3)>  {\"p\", [],[ \"#elixir\"], %{}}, {\"bold\", [],[ \"#elixir\"], %{}},\n      ...(3)>  {\"ol\", [], [{\"li\", [],[ \"#elixir\"], %{}}, {\"p\", [],[ \"elixir\"], %{}}, {\"p\", [], [\"#elixir\"], %{}}], %{}}\n      ...(3)> ]\n      ...(3)> map_ast_with(ast, false, transformer)\n      {[\n       {\"p\", [],[{\"a\", [{\"href\", \"https://elixir-lang.org\"}], [\"Elixir\"], %{}}], %{}}, {\"bold\", [],[ \"#elixir\"], %{}},\n       {\"ol\", [], [{\"li\", [],[ \"#elixir\"], %{}}, {\"p\", [],[ \"elixir\"], %{}}, {\"p\", [], [{\"a\", [{\"href\", \"https://elixir-lang.org\"}], [\"Elixir\"], %{}}], %{}}], %{}}\n      ], false}\n\nAn alternate, maybe more elegant solution would be to change the mapper function during AST traversal\nas demonstrated [here](https://github.com/pragdave/earmark/blob/master/test/acceptance/transform/map_ast_with_fnchange_test.exs)\n\n##","ref":"Earmark.Transform.html#module-structure-conserving-transformers"},{"type":"module","title":"Postprocessors and Convenience Functions - Earmark.Transform","doc":"These can be declared in the fields `postprocessor` and `registered_processors` in the `Options` struct,\n`postprocessor` is prepened to `registered_processors` and they are all applied to non string nodes (that\nis the quadtuples of the AST which are of the form `{tag, atts, content, meta}`\n\nAll postprocessors can just be functions on nodes or a `TagSpecificProcessors` struct which will group\nfunction applications depending on tags, as a convienience tuples of the form `{tag, function}` will be\ntransformed into a `TagSpecificProcessors` struct.\n\n    iex(4)> add_class1 = &Earmark.AstTools.merge_atts_in_node(&1, class: \"class1\")\n    ...(4)> m1 = Earmark.Options.make_options!(postprocessor: add_class1) |> make_postprocessor()\n    ...(4)> m1.({\"a\", [], nil, nil})\n    {\"a\", [{\"class\", \"class1\"}], nil, nil}\n\nWe can also use the `registered_processors` field:\n\n    iex(5)> add_class1 = &Earmark.AstTools.merge_atts_in_node(&1, class: \"class1\")\n    ...(5)> m2 = Earmark.Options.make_options!(registered_processors: add_class1) |> make_postprocessor()\n    ...(5)> m2.({\"a\", [], nil, nil})\n    {\"a\", [{\"class\", \"class1\"}], nil, nil}\n\nKnowing that values on the same attributes are added onto the front the following doctest demonstrates\nthe order in which the processors are executed\n\n    iex(6)> add_class1 = &Earmark.AstTools.merge_atts_in_node(&1, class: \"class1\")\n    ...(6)> add_class2 = &Earmark.AstTools.merge_atts_in_node(&1, class: \"class2\")\n    ...(6)> add_class3 = &Earmark.AstTools.merge_atts_in_node(&1, class: \"class3\")\n    ...(6)> m = Earmark.Options.make_options!(postprocessor: add_class1, registered_processors: [add_class2, {\"a\", add_class3}])\n    ...(6)> |> make_postprocessor()\n    ...(6)> [{\"a\", [{\"class\", \"link\"}], nil, nil}, {\"b\", [], nil, nil}]\n    ...(6)> |> Enum.map(m)\n    [{\"a\", [{\"class\", \"class3 class2 class1 link\"}], nil, nil}, {\"b\", [{\"class\", \"class2 class1\"}], nil, nil}]\n\nWe can see that the tuple form has been transformed into a tag specific transformation **only** as a matter of fact, the explicit definition would be:\n\n    iex(7)> m = make_postprocessor(\n    ...(7)>   %Earmark.Options{\n    ...(7)>     registered_processors:\n    ...(7)>       [Earmark.TagSpecificProcessors.new({\"a\", &Earmark.AstTools.merge_atts_in_node(&1, target: \"_blank\")})]})\n    ...(7)> [{\"a\", [{\"href\", \"url\"}], nil, nil}, {\"b\", [], nil, nil}]\n    ...(7)> |> Enum.map(m)\n    [{\"a\", [{\"href\", \"url\"}, {\"target\", \"_blank\"}], nil, nil}, {\"b\", [], nil, nil}]\n\nWe can also define a tag specific transformer in one step, which might (or might not) solve potential performance issues\nwhen running too many processors\n\n    iex(8)> add_class4 = &Earmark.AstTools.merge_atts_in_node(&1, class: \"class4\")\n    ...(8)> add_class5 = &Earmark.AstTools.merge_atts_in_node(&1, class: \"class5\")\n    ...(8)> add_class6 = &Earmark.AstTools.merge_atts_in_node(&1, class: \"class6\")\n    ...(8)> tsp = Earmark.TagSpecificProcessors.new([{\"a\", add_class5}, {\"b\", add_class5}])\n    ...(8)> m = Earmark.Options.make_options!(\n    ...(8)>       postprocessor: add_class4,\n    ...(8)>       registered_processors: [tsp, add_class6])\n    ...(8)> |> make_postprocessor()\n    ...(8)> [{\"a\", [], nil, nil}, {\"c\", [], nil, nil}, {\"b\", [], nil, nil}]\n    ...(8)> |> Enum.map(m)\n    [{\"a\", [{\"class\", \"class6 class5 class4\"}], nil, nil}, {\"c\", [{\"class\", \"class6 class4\"}], nil, nil}, {\"b\", [{\"class\", \"class6 class5 class4\"}], nil, nil}]\n\nOf course the mechanics shown above is hidden if all we want is to trigger the postprocessor chain in `Earmark.as_html`, here goes a typical\nexample\n\n    iex(9)> add_target = fn node -> # This will only be applied to nodes as it will become a TagSpecificProcessors\n    ...(9)>   if Regex.match?(~r{\\.x\\.com\\z}, Earmark.AstTools.find_att_in_node(node, \"href\", \"\")), do:\n    ...(9)>     Earmark.AstTools.merge_atts_in_node(node, target: \"_blank\"), else: node end\n    ...(9)> options = [\n    ...(9)> registered_processors: [{\"a\", add_target}, {\"p\", &Earmark.AstTools.merge_atts_in_node(&1, class: \"example\")}]]\n    ...(9)> markdown = [\n    ...(9)>   \"http://hello.x.com\",\n    ...(9)>   \"\",\n    ...(9)>   \"[some](url)\",\n    ...(9)>  ]\n    ...(9)> Earmark.as_html!(markdown, options)\n    \" \\n http://hello.x.com  \\n \\n some  \\n\"\n\n###","ref":"Earmark.Transform.html#module-postprocessors-and-convenience-functions"},{"type":"module","title":"Use case: Modification of Link Attributes depending on the URL - Earmark.Transform","doc":"This would be done as follows\n\n```elixir\n        Earmark.as_html!(markdown, registered_processors: {\"a\", my_function_that_is_invoked_only_with_a_nodes})\n```\n\n###","ref":"Earmark.Transform.html#module-use-case-modification-of-link-attributes-depending-on-the-url"},{"type":"module","title":"Use case: Modification of the AST according to Annotations - Earmark.Transform","doc":"**N.B.** Annotation are an _experimental_ feature in 1.4.16-pre and are documented [here](https://github.com/RobertDober/earmark_parser/#annotations)\n\nBy annotating our markdown source we can then influence the rendering. In this example we will just\nadd some decoration\n\n    iex(10)> markdown = [ \"A joke %% smile\", \"\", \"Charming %% in_love\" ]\n    ...(10)> add_smiley = fn {_, _, _, meta} = quad, _acc ->\n    ...(10)>                case Map.get(meta, :annotation) do\n    ...(10)>                  \"%% smile\"   -> {quad, \"\\u1F601\"}\n    ...(10)>                  \"%% in_love\" -> {quad, \"\\u1F60d\"}\n    ...(10)>                  _            -> {quad, nil}\n    ...(10)>                end\n    ...(10)>                text, nil -> {text, nil}\n    ...(10)>                text, ann -> {\"#{text} #{ann}\", nil}\n    ...(10)>              end\n    ...(10)> Earmark.as_ast!(markdown, annotations: \"%%\") |> Earmark.Transform.map_ast_with(nil, add_smiley) |> Earmark.transform\n    \" \\nA joke  ὠ1 \\n \\nCharming  ὠd \\n\"\n\n##","ref":"Earmark.Transform.html#module-use-case-modification-of-the-ast-according-to-annotations"},{"type":"module","title":"Structure Modifying Transformers - Earmark.Transform","doc":"For structure modifications a tree traversal is needed and no clear pattern of how to assist this task with\ntools has emerged yet.","ref":"Earmark.Transform.html#module-structure-modifying-transformers"},{"type":"function","title":"Earmark.Transform.make_postprocessor/1","doc":"","ref":"Earmark.Transform.html#make_postprocessor/1"},{"type":"function","title":"Earmark.Transform.map_ast/3","doc":"This is a structure conserving transformation\n\n    iex(11)> {:ok, ast, _} = Earmark.Parser.as_ast(\"- one\\n- two\\n\")\n    ...(11)> map_ast(ast, &(&1))\n    [{\"ul\", [], [{\"li\", [], [\"one\"], %{}}, {\"li\", [], [\"two\"], %{}}], %{}}]\n\nA more useful transformation\n    iex(12)> {:ok, ast, _} = Earmark.Parser.as_ast(\"- one\\n- two\\n\")\n    ...(12)> fun = fn {_, _, _, _}=n -> Earmark.AstTools.merge_atts_in_node(n, class: \"private\")\n    ...(12)>           string      -> string end\n    ...(12)> map_ast(ast, fun)\n    [{\"ul\", [{\"class\", \"private\"}], [{\"li\", [{\"class\", \"private\"}], [\"one\"], %{}}, {\"li\", [{\"class\", \"private\"}], [\"two\"], %{}}], %{}}]\n\nHowever the usage of the `ignore_strings` option renders the code much simpler\n\n    iex(13)> {:ok, ast, _} = Earmark.Parser.as_ast(\"- one\\n- two\\n\")\n    ...(13)> map_ast(ast, &Earmark.AstTools.merge_atts_in_node(&1, class: \"private\"), true)\n    [{\"ul\", [{\"class\", \"private\"}], [{\"li\", [{\"class\", \"private\"}], [\"one\"], %{}}, {\"li\", [{\"class\", \"private\"}], [\"two\"], %{}}], %{}}]","ref":"Earmark.Transform.html#map_ast/3"},{"type":"function","title":"Earmark.Transform.map_ast_with/4","doc":"This too is a structure perserving transformation but a value is passed to the mapping function as an accumulator, and the mapping\nfunction needs to return the new node and the accumulator as a tuple, here is a simple example\n\n    iex(14)> {:ok, ast, _} = Earmark.Parser.as_ast(\"- 1\\n\\n2\\n- 3\\n\")\n    ...(14)> summer = fn {\"li\", _, [v], _}=n, s -> {v_, _} = Integer.parse(v); {n, s + v_}\n    ...(14)>             n, s -> {n, s} end\n    ...(14)> map_ast_with(ast, 0, summer, true)\n    {[{\"ul\", [], [{\"li\", [], [\"1\"], %{}}], %{}}, {\"p\", [], [\"2\"], %{}}, {\"ul\", [], [{\"li\", [], [\"3\"], %{}}], %{}}], 4}\n\nor summing all numbers\n\n    iex(15)> {:ok, ast, _} = Earmark.Parser.as_ast(\"- 1\\n\\n2\\n- 3\\n\")\n    ...(15)> summer = fn {_, _, _, _}=n, s -> {n, s}\n    ...(15)>             n, s -> {n_, _} = Integer.parse(n); {\"*\", s+n_} end\n    ...(15)> map_ast_with(ast, 0, summer)\n    {[{\"ul\", [], [{\"li\", [], [\"*\"], %{}}], %{}}, {\"p\", [], [\"*\"], %{}}, {\"ul\", [], [{\"li\", [], [\"*\"], %{}}], %{}}], 6}","ref":"Earmark.Transform.html#map_ast_with/4"},{"type":"function","title":"Earmark.Transform.transform/2","doc":"Transforms an AST to html, also accepts the result of `map_ast_with` for convenience","ref":"Earmark.Transform.html#transform/2"},{"type":"module","title":"Earmark.Transform.Pop","doc":"A wrapper struct that wraps a mapping function and is pushed onto the\nstack for traversal. This not only allows to pop from the stack onto\nthe result but to store an older mapper function for usage of a replacement\nmapper function for the subtree traversal","ref":"Earmark.Transform.Pop.html"}],"content_type":"text/markdown"}